#pragma once

#include <SDL.h>
#include <string>
#include <vector>

#include "color.h"
#include "utils/bit_array.h"

namespace Foundation
{
	// Forward declarations
	class IManaged;
	class TextField;
	class DrawField;
	class Image;

	// Class
	class Viewport final
	{
	public:
		Viewport(int inWidth, int inHeight, const std::string& inCaption);
		~Viewport();

		int GetWidth() const;
		int GetHeight() const;

		void SetAdditionTitleInfo(const std::string& inAdditionTitleInfo);

		void Begin();
		void End();

		TextField* CreateTextField(unsigned inWidth, unsigned int inHeight, int inX, int inY);
		DrawField* CreateDrawField(unsigned inWidth, unsigned int inHeight, int inX, int inY);
		Image* CreateImageFromFile(const std::string& inFileName);
		Image* CreateImageFromARGBData(void* inData, unsigned int inWidth, unsigned int inHeight, bool inIncludeAlphaChannel);

		void Destroy(IManaged* inManagedResource);

		const Palette& GetPalette() const;
		SDL_Window* GetSDLWindow() const;
		
	private:
		const int m_ResolutionX;
		const int m_ResolutionY;

		Palette m_Palette;

		SDL_Window* m_Window;
		SDL_Renderer* m_Renderer;

		std::string m_Caption;

		std::vector<IManaged*> m_ManagedResources;
	};
}