#include "keyboard_utils.h"
#include "keyboard.h"

namespace Foundation
{
	namespace KeyboardUtils
	{
		bool IsModifierExclusivelyDown(int inModifierMask, int inModifiersToCheckAgainst)
		{
			if ((inModifierMask & ~inModifiersToCheckAgainst) != 0)
				return false;
			if (static_cast<bool>(inModifiersToCheckAgainst & Keyboard::Shift) != static_cast<bool>(inModifierMask & Keyboard::Shift))
				return false;
			if (static_cast<bool>(inModifiersToCheckAgainst & Keyboard::Control) != static_cast<bool>(inModifierMask & Keyboard::Control))
				return false;
			if (static_cast<bool>(inModifiersToCheckAgainst & Keyboard::Alt) != static_cast<bool>(inModifierMask & Keyboard::Alt))
				return false;

			return true;
		}


		bool IsAcceptableInputText(char inCharacter)
		{
			static char acceptable_characters[] = " 0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ.,-+_!\"#&%()[]//\\";

			for (int i = 0; i < sizeof(acceptable_characters); ++i)
			{
				if (acceptable_characters[i] == inCharacter)
					return true;
			}

			return false;
		}


		bool IsAcceptableInputFilename(char inCharacter)
		{
			static char acceptable_characters[] = " 0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ.,-+_!()[]";

			for (int i = 0; i < sizeof(acceptable_characters); ++i)
			{
				if (acceptable_characters[i] == inCharacter)
					return true;
			}

			return false;
		}


		char FilterDecimalDigit(const SDL_Keycode inKeyCode)
		{
			if (inKeyCode >= SDLK_0 && inKeyCode <= SDLK_9)
				return '0' + (inKeyCode - SDLK_0);
			if (inKeyCode >= SDLK_KP_1 && inKeyCode <= SDLK_KP_9)
				return '1' + (inKeyCode - SDLK_KP_1);
			if (inKeyCode == SDLK_KP_0)
				return '0';

			return 0;
		}


		char FilterHexDigit(const SDL_Keycode inKeyCode)
		{
			if (inKeyCode >= SDLK_a && inKeyCode <= SDLK_f)
				return 'a' + (inKeyCode - SDLK_a);
			return FilterDecimalDigit(inKeyCode);
		}


		char FilterLetter(const SDL_Keycode inKeyCode)
		{
			if (inKeyCode >= SDLK_a && inKeyCode <= SDLK_z)
				return 'a' + (inKeyCode - SDLK_a);

			return 0;
		}


		char FilterLetterWithModifier(const SDL_Keycode inKeyCode, const int inModifierMask)
		{
			if ((inModifierMask & ~Keyboard::Shift) == 0)
			{
				if (inKeyCode >= SDLK_a && inKeyCode <= SDLK_z)
				{
					if ((inModifierMask & Keyboard::Shift) == 0)
						return 'a' + (inKeyCode - SDLK_a);
					else
						return 'A' + (inKeyCode - SDLK_a);
				}
				else if (inKeyCode == SDLK_SPACE)
					return ' ';
			}

			return 0;
		}

		char FilterSpecialCharacters(const SDL_Keycode inKeyCode, const int inModifierMask)
		{
			//if (inModifierMask == 0)
			{
				switch (inKeyCode)
				{
				case SDLK_MINUS:
					return '-';
				case SDLK_UNDERSCORE:
					return '_';
				case SDLK_LEFTPAREN:
					return '(';
				case SDLK_RIGHTPAREN:
					return ')';
				case SDLK_PERIOD:
					return '.';
				}
			}

			return 0;
		}
	}
}