#include "platform_sdl_linux.h"
#include "mutex_sdl.h"

#ifdef _SF2_LINUX
#include "libraries/ghc/fs_std.h"
#include <assert.h>
#include <system_error>
#include <pwd.h>
#include <unistd.h>

#include <stdio.h>

using namespace fs;

namespace Foundation
{
	PlatformSDLLinux::PlatformSDLLinux(int argc, char* argv[])
	{
        std::error_code error_code;
        char buffer[PATH_MAX];

        // Standard mount points for external Drives
        m_LogicalDrivesList.push_back("/mnt");
        m_LogicalDrivesList.push_back("/Media");

        // Add the user's home folder
        char *realHome = getpwuid(getuid())->pw_dir;

        if(realHome != nullptr)
        {
            m_RealHome = std::string(realHome);
            
            m_LogicalDrivesList.push_back(realHome);
            
            // Add some standard folders that user might want to use...
            m_LogicalDrivesList.push_back(std::string(realHome) + "/Desktop");
            m_LogicalDrivesList.push_back(std::string(realHome) + "/Music");
            m_LogicalDrivesList.push_back(std::string(realHome) + "/Documents");
        }

        // let's also add the current dir
        fs::path application_path = getcwd(buffer, PATH_MAX);
        m_LogicalDrivesList.push_back(std::string(buffer));

        if(!fs::is_directory(application_path))
            fs::current_path(application_path.remove_filename());
        else
            fs::current_path(application_path);//, error_code);
    }


	PlatformSDLLinux::~PlatformSDLLinux()
	{

	}

	//---------------------------------------------------------------------------------------

    bool PlatformSDLLinux::Storage_SetCurrentPath(const std::string& inPath) const
    {
        std::error_code ec;
        fs::path previous_path = fs::current_path(ec);

        if (!ec)
        {
            // Set current path, if that fails, set the prevcious path again
            fs::current_path(inPath, ec);
            if (ec)
            {
                fs::current_path(previous_path, ec);
                return false;
            }
            
            // Try get the current path to verify that it works (which will pop up a dialog on macOS for instance, if there are special enforcements to consider. If getting the path fails, set the previous again.
            auto cp = fs::current_path(ec);
            
            fs::directory_iterator(cp, ec);
            if(!ec)
                return true;
            
            fs::current_path(previous_path, ec);
        }

        return false;
    }

    unsigned int PlatformSDLLinux::Storage_GetLogicalDrivesCount() const
	{
		return static_cast<unsigned int>(m_LogicalDrivesList.size());
	}

	std::string PlatformSDLLinux::Storage_GetLogicalDriveName(unsigned int inLogicalDrive) const
	{
		assert(inLogicalDrive < m_LogicalDrivesList.size());
		return m_LogicalDrivesList[inLogicalDrive];
	}

    bool PlatformSDLLinux::Storage_IsSystemFile(const std::string& inPath) const
    {
        fs::path path(inPath);
        
        // Remove any filename that starts with a dot
        std::string filename = path.filename().string();
        if(filename != "." && filename != ".." && filename[0] == '.')
            return true;
        
        return false;
    }

    std::string PlatformSDLLinux::Storage_GetStartupPath() const
    {
        return m_RealHome;
    }
}

#endif 

