#include "platform_sdl_macos.h"
#include "mutex_sdl.h"

#ifndef _SF2_WINDOWS
#include "libraries/ghc/fs_std.h"
#include <assert.h>
#include <mach-o/dyld.h>
#include <system_error>
#include <pwd.h>

using namespace fs;

namespace Foundation
{
	PlatformSDLMacOS::PlatformSDLMacOS(int argc, char* argv[])
	{
        std::error_code error_code;
        
        uint32_t buffer_size = PATH_MAX;
        _NSGetExecutablePath(m_ApplicationPath, &buffer_size);

        m_LogicalDrivesList.push_back("/Volumes");
        
        // Add the user's home folder
        char *realHome = getpwuid(getuid())->pw_dir;
        
        if(realHome != nullptr)
        {
            m_RealHome = std::string(realHome);
            
            m_LogicalDrivesList.push_back(realHome);
            
            // Add some standard folders that user might want to use...
            m_LogicalDrivesList.push_back(std::string(realHome) + "/Desktop");
            m_LogicalDrivesList.push_back(std::string(realHome) + "/Music");
            m_LogicalDrivesList.push_back(std::string(realHome) + "/Documents");
        }

        path application_path = m_ApplicationPath;
        
        if(!fs::is_directory(application_path))
            current_path(application_path.remove_filename());
        else
            current_path(application_path);//, error_code);
    }


	PlatformSDLMacOS::~PlatformSDLMacOS()
	{

	}

	//---------------------------------------------------------------------------------------

    bool PlatformSDLMacOS::Storage_SetCurrentPath(const std::string& inPath) const
    {
        std::error_code ec;
        fs::path previous_path = fs::current_path(ec);

        if (!ec)
        {
            // Set current path, if that fails, set the prevcious path again
            fs::current_path(inPath, ec);
            if (ec)
            {
                fs::current_path(previous_path, ec);
                return false;
            }
            
            // Try get the current path to verify that it works (which will pop up a dialog on macOS for instance, if there are special enforcements to consider. If getting the path fails, set the previous again.
            auto cp = fs::current_path(ec);
            
            fs::directory_iterator(cp, ec);
            if(!ec)
                return true;
            
            fs::current_path(previous_path, ec);
        }

        return false;
    }

    unsigned int PlatformSDLMacOS::Storage_GetLogicalDrivesCount() const
	{
		return static_cast<unsigned int>(m_LogicalDrivesList.size());
	}

	std::string PlatformSDLMacOS::Storage_GetLogicalDriveName(unsigned int inLogicalDrive) const
	{
		assert(inLogicalDrive < m_LogicalDrivesList.size());
		return m_LogicalDrivesList[inLogicalDrive];
	}

    bool PlatformSDLMacOS::Storage_IsSystemFile(const std::string& inPath) const
    {
        fs::path path(inPath);
        
        // Remove any filename that starts with a dot
        std::string filename = path.filename().string();
        if(filename != "." && filename != ".." && filename[0] == '.')
            return true;
        
        return false;
    }

    std::string PlatformSDLMacOS::Storage_GetStartupPath() const
    {
        return m_RealHome;
    }
}

#endif 

