#include "runtime/editor/keys/keyhook_setup.h"
#include "foundation/input/keyboard.h"

using namespace Foundation;

namespace Editor
{
	KeyHookSetup::KeyHookSetup()
	{
		ConfigureStore();
		ApplyOverrides();
	}


	Utility::KeyHookStore& KeyHookSetup::GetKeyHookStore()
	{
		return m_KeyHookStore;
	}


	void KeyHookSetup::ConfigureStore()
	{
		std::vector<Utility::KeyHookStore::HookDefinition> definitions;

		definitions.push_back({ "ScreenEdit.Stop", { SDLK_ESCAPE, Keyboard::None } });
		definitions.push_back({ "ScreenEdit.Play", { SDLK_F1, Keyboard::None } });
		definitions.push_back({ "ScreenEdit.PlayFromMarker", { SDLK_F2, Keyboard::None } });
		definitions.push_back({ "ScreenEdit.PlayFromLast", { SDLK_F2, Keyboard::Shift } });
		definitions.push_back({ "ScreenEdit.PlayFromCursor", { SDLK_F2, Keyboard::Control } });
		definitions.push_back({ "ScreenEdit.OctaveDown", { SDLK_F3, Keyboard::None } });
		definitions.push_back({ "ScreenEdit.OctaveUp", { SDLK_F4, Keyboard::None } });
		definitions.push_back({ "ScreenEdit.OpenUtilitiesDialog", { SDLK_F6, Keyboard::None } });
		definitions.push_back({ "ScreenEdit.OpenOptionsDialog", { SDLK_F6, Keyboard::Control } });
		definitions.push_back({ "ScreenEdit.ToggleSIDModel", { SDLK_F9, Keyboard::None } });
		definitions.push_back({ "ScreenEdit.ToggleRegion", { SDLK_F9, Keyboard::Shift } });
		definitions.push_back({ "ScreenEdit.LoadSong", { SDLK_F10, Keyboard::None } });
		definitions.push_back({ "ScreenEdit.LoadInstrument", { SDLK_F10, Keyboard::Shift } });
		definitions.push_back({ "ScreenEdit.ImportSong", { SDLK_F10, Keyboard::Control } });
		definitions.push_back({ "ScreenEdit.SaveSong", { SDLK_F11, Keyboard::None } });
		definitions.push_back({ "ScreenEdit.SaveInstrument", { SDLK_F11, Keyboard::Shift } });
		definitions.push_back({ "ScreenEdit.ToggleFlightRecorderOverlay", { SDLK_d, Keyboard::Control } });
		definitions.push_back({ "ScreenEdit.ToggleDebugView", { SDLK_d, Keyboard::Shift | Keyboard::Alt } });
		definitions.push_back({ "ScreenEdit.ToggleMuteChannel1", { SDLK_1, Keyboard::Control } });
		definitions.push_back({ "ScreenEdit.ToggleMuteChannel2", { SDLK_2, Keyboard::Control } });
		definitions.push_back({ "ScreenEdit.ToggleMuteChannel3", { SDLK_3, Keyboard::Control } });
		definitions.push_back({ "ScreenEdit.SetMarker", { SDLK_m, Keyboard::Control } });
		definitions.push_back({ "ScreenEdit.GotoMarker", { SDLK_g, Keyboard::Control } });
		definitions.push_back({ "ScreenEdit.QuickSave", { SDLK_s, Keyboard::Control } });
		definitions.push_back({ "ScreenEdit.Undo", { SDLK_z, Keyboard::Control } });
		definitions.push_back({ "ScreenEdit.Redo", { SDLK_x, Keyboard::Control } });
		definitions.push_back({ "ScreenEdit.RedoAlternate", { SDLK_y, Keyboard::Control } });
		definitions.push_back({ "ScreenEdit.DecrementEventHighlightInterval", { SDLK_UP, Keyboard::Alt } });
		definitions.push_back({ "ScreenEdit.DecrementEventHighlightOffset", { SDLK_UP, Keyboard::Alt | Keyboard::Shift } });
		definitions.push_back({ "ScreenEdit.IncrementEventHighlightInterval", { SDLK_DOWN, Keyboard::Alt } });
		definitions.push_back({ "ScreenEdit.IncrementEventHighlightOffset", { SDLK_DOWN, Keyboard::Alt | Keyboard::Shift } });
		definitions.push_back({ "ScreenEdit.ToggleFollowPlay", { SDLK_p, Keyboard::Control } });
		definitions.push_back({ "ScreenEdit.ToggleDisplayHexUppercase", { SDLK_u, Keyboard::Control } });
		definitions.push_back({ "ScreenEdit.IncrementInstrumentIndex", { SDLK_KP_PLUS, Keyboard::None } });
		definitions.push_back({ "ScreenEdit.DecrementInstrumentIndex", { SDLK_KP_MINUS, Keyboard::None } });
		definitions.push_back({ "ScreenEdit.IncrementInstrumentIndex_Alternative", { SDLK_DOWN, Keyboard::Alt | Keyboard::Control } });
		definitions.push_back({ "ScreenEdit.DecrementInstrumentIndex_Alternative", { SDLK_UP, Keyboard::Alt | Keyboard::Control } });
		definitions.push_back({ "ScreenEdit.IncrementCommandIndex", { SDLK_KP_PLUS, Keyboard::Control } });
		definitions.push_back({ "ScreenEdit.DecrementCommandIndex", { SDLK_KP_MINUS, Keyboard::Control } });

		definitions.push_back({ "Track.CursorUp", { SDLK_UP, Keyboard::None } });
		definitions.push_back({ "Track.CursorDown", { SDLK_DOWN, Keyboard::None } });
		definitions.push_back({ "Track.ToggleGateUntilNextEvent", { SDLK_DOWN, Keyboard::Shift | Keyboard::Control } });
		definitions.push_back({ "Track.ToggleGateSincePreviousEvent", { SDLK_UP, Keyboard::Shift | Keyboard::Control } });
		definitions.push_back({ "Track.PageUp", { SDLK_PAGEUP, Keyboard::None } });
		definitions.push_back({ "Track.PageDown", { SDLK_PAGEDOWN, Keyboard::None } });
		definitions.push_back({ "Track.TransposeSemiToneDown", { SDLK_F3, Keyboard::Shift } });
		definitions.push_back({ "Track.TransposeOctaveDown", { SDLK_F3, Keyboard::Control } });
		definitions.push_back({ "Track.TransposeSemiToneUp", { SDLK_F4, Keyboard::Shift } });
		definitions.push_back({ "Track.TransposeOctaveUp", { SDLK_F4, Keyboard::Control } });
		definitions.push_back({ "Track.InsertEventsInSequence", { SDLK_F5, Keyboard::Shift } });
		definitions.push_back({ "Track.ResizeSequence", { SDLK_F5, Keyboard::None } });
		definitions.push_back({ "Track.EraseEventUnderCursor", { SDLK_SPACE, Keyboard::None } });
		definitions.push_back({ "Track.EraseEventLine", { SDLK_SPACE, Keyboard::Control } });
		definitions.push_back({ "Track.ToggleGateUnderCursor", { SDLK_SPACE, Keyboard::Shift } });
		definitions.push_back({ "Track.CursorLeft", { SDLK_LEFT, Keyboard::None } });
		definitions.push_back({ "Track.CursorRight", { SDLK_RIGHT, Keyboard::None } });
		definitions.push_back({ "Track.CursorLeftColumn", { SDLK_LEFT, Keyboard::Control } });
		definitions.push_back({ "Track.CursorRightColumn", { SDLK_RIGHT, Keyboard::Control } });
		definitions.push_back({ "Track.Insert", { SDLK_INSERT, Keyboard::None } });
		definitions.push_back({ "Track.InsertAndGrow", { SDLK_INSERT, Keyboard::Control } });
		definitions.push_back({ "Track.Delete", { SDLK_DELETE, Keyboard::None } });
		definitions.push_back({ "Track.InsertAndShrink", { SDLK_DELETE, Keyboard::Control } });
		definitions.push_back({ "Track.Backspace", { SDLK_BACKSPACE, Keyboard::None } });
		definitions.push_back({ "Track.BackspaceAndShrink", { SDLK_BACKSPACE, Keyboard::Control } });
		definitions.push_back({ "Track.ShiftBackspace", { SDLK_BACKSPACE, Keyboard::Shift } });
		definitions.push_back({ "Track.ShiftBackspaceAndGrow", { SDLK_BACKSPACE, Keyboard::Shift | Keyboard::Control } });
		definitions.push_back({ "Track.Home", { SDLK_HOME, Keyboard::None } });
		definitions.push_back({ "Track.End", { SDLK_END, Keyboard::None } });
		definitions.push_back({ "Track.ApplyOrderlistInput", { SDLK_RETURN, Keyboard::None } });
		definitions.push_back({ "Track.CancelOrderlistInput", { SDLK_ESCAPE, Keyboard::None } });
		definitions.push_back({ "Track.ToggleTieNote", { SDLK_RETURN, Keyboard::Shift } });
		definitions.push_back({ "Track.Copy", { SDLK_c, Keyboard::Control } });
		definitions.push_back({ "Track.Paste", { SDLK_v, Keyboard::Control } });
		definitions.push_back({ "Track.InsertFirstFreeSequence", { SDLK_f, Keyboard::Control } });
		definitions.push_back({ "Track.SplitSequenceAtEventPosition", { SDLK_b, Keyboard::Control } });
		definitions.push_back({ "Track.SetSelectedInstrumentIndexValue", { SDLK_i, Keyboard::Control } });
		definitions.push_back({ "Track.SetSelectedCommandIndexValue", { SDLK_o, Keyboard::Control } });
		definitions.push_back({ "Track.Debug", { SDLK_d, Keyboard::Alt } });

		m_KeyHookStore.PassBaseDefinitions(definitions);
	}

	void KeyHookSetup::ApplyOverrides()
	{
#ifndef _SF2_WINDOWS
		m_KeyHookStore.OverrideDefinition({ "ScreenEdit.PlayFromCursor", { SDLK_F2, Keyboard::Cmd } });
		m_KeyHookStore.OverrideDefinition({ "Track.TransposeOctaveDown", { SDLK_F3, Keyboard::Cmd | Keyboard::Shift } });
		m_KeyHookStore.OverrideDefinition({ "Track.TransposeOctaveUp", { SDLK_F4, Keyboard::Cmd | Keyboard::Shift } });
		m_KeyHookStore.OverrideDefinition({ "Track.CursorLeftColumn", { SDLK_LEFT, Keyboard::Cmd } });
		m_KeyHookStore.OverrideDefinition({ "Track.CursorRightColumn", { SDLK_RIGHT, Keyboard::Cmd } });
		m_KeyHookStore.OverrideDefinition({ "ScreenEdit.IncrementInstrumentIndex", { SDLK_EQUALS, Keyboard::Cmd } });
		m_KeyHookStore.OverrideDefinition({ "ScreenEdit.DecrementInstrumentIndex", { SDLK_MINUS, Keyboard::Cmd } });
		m_KeyHookStore.OverrideDefinition({ "ScreenEdit.IncrementCommandIndex", { SDLK_EQUALS, Keyboard::Cmd | Keyboard::Shift } });
		m_KeyHookStore.OverrideDefinition({ "ScreenEdit.DecrementCommandIndex", { SDLK_MINUS, Keyboard::Cmd | Keyboard::Shift } });
#endif //_SF2_WINDOWS
	}
}
