#include "utils/configfile.h"
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>


namespace Utility
{
	ConfigFile::ConfigFile(const std::string& inFilename)
	{
		FILE* fp = fopen(inFilename.c_str(), "r");
		if (fp != nullptr)
		{
			bool reading = true;

			while (reading)
			{
				char character;
				std::string line;

				while (true)
				{
					size_t bytes_read = fread(&character, 1, 1, fp);

					if (bytes_read != 1)
						reading = false;

					if (character == '\0' || !reading)
					{
						if(!line.empty())
							ParseLine(line);

						break;
					}
				}
			}

			fclose(fp);

		}
	}

	bool ConfigFile::HasConfiguration(const std::string& inKey) const
	{
		return m_ConfigTokensMap.find(inKey) != m_ConfigTokensMap.end();
	}

	const std::string ConfigFile::GetConfiguration(const std::string& inKey) const
	{
		assert(HasConfiguration(inKey));
		return m_ConfigTokensMap.find(inKey)->second;
	}

	void ConfigFile::ParseLine(const std::string& inLine)
	{
		if (inLine.empty())
			return;

		size_t index = inLine.find('=');
		if (index > 0)
		{
			std::string first = inLine.substr(0, index - 1);
			std::string second = inLine.substr(index + 1, inLine.length() - (index + 1));

			// TODO: Trim the strings first!
			assert(!HasConfiguration(first));
			m_ConfigTokensMap[first] = second;
		}
	}
}